@@ -9,6 +9,7 @@
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <stdarg.h>
+#include <math.h>
 
 #ifdef HAVE_XFT2
 #include <gdk/gdkx.h>
@@ -202,8 +203,8 @@ setup_font_sample (GtkWidget   *darea,
 	const char *string1 = "abcfgop AO ";
 	const char *string2 = "abcfgop";
 
-	XftColor black = { 0, {      0,      0,       0, 0xffff } };
-	XftColor white = { 0, { 0xffff, 0xffff,  0xffff, 0xffff } };
+	XftColor black, white;
+	XRenderColor rendcolor;
 
 	Display *xdisplay = gdk_x11_get_default_xdisplay ();
 
@@ -265,6 +266,17 @@ setup_font_sample (GtkWidget   *darea,
 
 	draw = XftDrawCreate (xdisplay, GDK_DRAWABLE_XID (pixmap), xvisual, xcolormap);
 
+	rendcolor.red = 0;
+	rendcolor.green = 0;
+	rendcolor.blue = 0;
+	rendcolor.alpha = 0xffff;
+	XftColorAllocValue (xdisplay, xvisual, xcolormap, &rendcolor, &black);
+	
+	rendcolor.red = 0xffff;
+	rendcolor.green = 0xffff;
+	rendcolor.blue = 0xffff;
+	rendcolor.alpha = 0xffff;
+	XftColorAllocValue (xdisplay, xvisual, xcolormap, &rendcolor, &white);
 	XftDrawRect (draw, &white, 0, 0, width, height);
 	if (font1)
 		XftDrawStringUtf8 (draw, &black, font1,
@@ -492,12 +504,16 @@ application_font_to_gconf (GConfPropertyEditor *peditor,
 		if (danger_level > MAX_FONT_POINT_WITHOUT_WARNING) {
 			warning_label = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
 							 _("Font may be too large"),
-							 _("The font selected is %d points large, and may make it difficult to effectively use the computer.  It is recommended that you select a size smaller than %d."));
+							 ngettext ("The font selected is %d point large, and may make it difficult to effectively use the computer.  It is recommended that you select a size smaller than %d.",
+								   "The font selected is %d points large, and may make it difficult to effectively use the computer.  It is recommended that you select a size smaller than %d.",
+								   danger_level));
 			warning_label2 = g_strdup_printf (warning_label, danger_level, MAX_FONT_POINT_WITHOUT_WARNING);
 		} else {
 			warning_label = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
 							 _("Font may be too large"),
-							 _("The font selected is %d points large, and may make it difficult to effectively use the computer.  It is recommended that you select a smaller sized font."));
+							 ngettext ("The font selected is %d point large, and may make it difficult to effectively use the computer.  It is recommended that you select a smaller sized font.",
+								   "The font selected is %d points large, and may make it difficult to effectively use the computer.  It is recommended that you select a smaller sized font.",
+								   danger_level));
 			warning_label2 = g_strdup_printf (warning_label, danger_level);
 		}
 		gtk_label_set_markup (GTK_LABEL (WID ("font_size_warning_label")), warning_label2);
@@ -734,6 +750,9 @@ dpi_load (GConfClient   *client,
 {
 	gdouble dpi = gconf_client_get_float (client, FONT_DPI_KEY, NULL);
 
+	if (dpi < 50.)
+		dpi = 50.;
+
 	in_change = TRUE;
 	gtk_spin_button_set_value (spinner, dpi);
 	in_change = FALSE;
@@ -792,6 +811,8 @@ cb_show_details (GtkWidget *button,
 		GladeXML *dialog = glade_xml_new (GLADEDIR "/font-properties.glade", "render_details", NULL);
 		GtkWidget *dpi_spinner;
 		GnomeVFSURI *uri;
+		int dpi;
+		GtkAdjustment *adjustment;
 
 		details_dialog = WID ("render_details");
 		uri = gnome_vfs_uri_new ("fonts:///");
@@ -805,6 +826,15 @@ cb_show_details (GtkWidget *button,
 		gtk_window_set_transient_for (GTK_WINDOW (details_dialog), parent);
 
 		dpi_spinner = WID ("dpi_spinner");
+
+		/* pick a sensible maximum dpi */
+		dpi = floor ((gdk_screen_width () / gdk_screen_width_mm () +
+			      gdk_screen_height () / gdk_screen_height_mm ()) * 25.4 / 2. + .5);
+		if (dpi < 50)
+			 dpi = 50; /* be extra careful */
+		adjustment = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (dpi_spinner));
+		adjustment->upper = dpi * 3;
+
 		dpi_load (client, GTK_SPIN_BUTTON (dpi_spinner));
 		g_signal_connect (dpi_spinner, "value_changed",
 				  G_CALLBACK (dpi_value_changed), client);
